Option Strict On

Imports System.diagnostics
Imports System.Threading

Public Class Logic
    Dim objData As New Datalayer
    Dim dtCurrentSignal As DataTable
    Dim Filename As String
    Dim Sampler As Thread
    Dim SamplerFinished As Boolean

#Region " New "
    Public Sub New()
        'Me.Filename = Filename
        Me.OpenDB()

        dtCurrentSignal = New DataTable("Signal")

        Dim MAC As DataColumn = New DataColumn("MAC")
        MAC.DataType = System.Type.GetType("System.String")
        dtCurrentSignal.Columns.Add(MAC)

        Dim Signal As DataColumn = New DataColumn("Signal")
        Signal.DataType = System.Type.GetType("System.Int32")
        dtCurrentSignal.Columns.Add(Signal)

        Dim Noise As DataColumn = New DataColumn("Noise")
        Noise.DataType = System.Type.GetType("System.Int32")
        dtCurrentSignal.Columns.Add(Noise)

        'Dim SNR As DataColumn = New DataColumn("SNR")
        'SNR.DataType = System.Type.GetType("System.Int32")
        'dtCurrentSignal.Columns.Add(SNR)
    End Sub
#End Region

#Region " Open / Close DB Connection "
    Public Sub OpenDB()
        objData.Open()
    End Sub

    Public Sub CloseDB()
        objData.Close()
    End Sub
#End Region

#Region " ReadSignal "
    Public Function ReadSignalTXT() As DataTable
        Dim SR As IO.StreamReader
        Dim dtRow As DataRow

        dtCurrentSignal.Rows.Clear()

        Try
            SR = IO.File.OpenText(Filename)

            Dim splitSentence() As String

            Do Until SR.Peek = -1
                splitSentence = SR.ReadLine.Split(" "c)


                dtRow = dtCurrentSignal.NewRow()

                dtRow("MAC") = splitSentence(0)
                dtRow("Signal") = splitSentence(1)
                dtRow("Noise") = splitSentence(2)
                'dtRow("SNR") = CInt(splitSentence(1)) - CInt(splitSentence(2))

                dtCurrentSignal.Rows.Add(dtRow)
            Loop

            SR.Close()

        Catch ex As Exception
            Console.WriteLine("Error opening file " & Filename & ex.Message)
        End Try

        Return dtCurrentSignal
    End Function

    Public Function ReadSignal() As DataTable
        dtCurrentSignal.Rows.Clear()

        Dim SQLstr As String = "SELECT * FROM CurrentSignal"
        dtCurrentSignal = objData.ExecuteQuery(SQLstr)

        Return dtCurrentSignal
    End Function
#End Region

#Region " SamplerRun "
    Public Sub SamplerRun()
        Dim Iterator As Integer
        Dim dtBestSignal As DataTable

        dtBestSignal = ReadSignal().Copy ' First time is the best signal

        Thread.Sleep(2000)

        Iterator = 2

        While Iterator <> 0
            dtCurrentSignal = ReadSignal().Copy
            If dtCurrentSignal.Rows.Count >= dtBestSignal.Rows.Count Then ' >= most current reading
                dtBestSignal = dtCurrentSignal.Copy
            End If
            Iterator -= 1
            Thread.Sleep(2000)
        End While

        dtCurrentSignal = dtBestSignal.Copy

        SamplerFinished = True
    End Sub
#End Region

#Region " StartSampling "
    Public Sub StartSampling()
        ' Read the signal several times, saves the best reading and puts it in the global variable
        Sampler = New Thread(New ThreadStart(AddressOf SamplerRun))
        Sampler.Start()
    End Sub
#End Region

#Region " AddLocation / AddFingerprint"
    Public Sub AddLocation(ByVal Name As String, ByVal LokType As String, ByVal LokFunk As String(), ByVal delAv As String)
        Dim lok_ID As Integer
        Dim SQLstr As String

        lok_ID = getLokID()
        lok_ID += 1

        SQLstr = "INSERT INTO Lokasjon (lok_ID, lokNavn, lokTypeNavn) VALUES (" _
        & lok_ID & ", '" & Name & "', '" & LokType & "')"
        objData.ExecuteNonQuery(SQLstr) 'Lager ny lokasjon

        Dim i As Integer
        For i = 0 To LokFunk.Length - 1
            SQLstr = "INSERT INTO LokFunk (funkNavn, lok_ID) VALUES ('" _
            & LokFunk(i) & "', " & lok_ID & ")"
            objData.ExecuteNonQuery(SQLstr) 'Fyller inn hvilke funksjoner lokasjonen har
        Next

        SQLstr = "INSERT INTO Relasjon (relType_ID, brukergruppe_ID, lok_ID_fra, lok_ID_til) VALUES ('del_av', 'Alle', " _
        & lok_ID & ", " & delAv & ")"
        objData.ExecuteNonQuery(SQLstr) 'Oppretter relasjon "del_av" med overodnet lokasjon

        'SQLstr = "INSERT INTO Kart (lok_ID, kartURL) VALUES (" & lok_ID & ", '" & kartURL & "')"
        'objData.ExecuteNonQuery(SQLstr) 'Legger inn URL for kart over lokasjon

    End Sub

    Public Sub AddFingerprint(ByVal lok_ID As String)
        Dim dtRow As DataRow
        Dim Sample_ID As Integer
        Dim SQLstr As String

        If Not chkFP(CInt(lok_ID)) Then 'Create new fingerprint if necessary
            SQLstr = "INSERT INTO PosFP (lok_ID) VALUES (" & lok_ID & ")"

            objData.ExecuteNonQuery(SQLstr)
        End If

        SQLstr = "SELECT MAX(sample_ID) FROM AP WHERE lok_ID =" & lok_ID
        Sample_ID = CInt(objData.ExecuteScalar(SQLstr))
        If Sample_ID.Equals(0) Then 'Get the right sample number
            Sample_ID = 1
        Else
            Sample_ID += 1
        End If

        For Each dtRow In dtCurrentSignal.Rows  'Record fingerprint sample
            SQLstr = "INSERT INTO AP (MAC, signal, noise, lok_ID, sample_ID) VALUES ('" _
            & CStr(dtRow("MAC")) & "'," & CStr(dtRow("Signal")) & ", " & CStr(dtRow("Noise")) _
            & ", " & lok_ID & ", " & Sample_ID & ")"

            objData.ExecuteNonQuery(SQLstr)
        Next
    End Sub

#End Region

#Region " Add/Edit Location Info"
    Public Sub addLocationInfo(ByVal infoart As String, ByVal brukergruppe As String, ByVal lokID As String, ByVal homepage As String, ByVal service As String)
        Dim sqlStr As String

        sqlStr = "INSERT INTO LokInfo (infoArtNavn, brukergruppe_ID, lok_ID, homepageURL, serviceURL) VALUES ('" _
        & infoart & "', '" & brukergruppe & "', " & lokID & ", '" & homepage & "', '" & service & "')"

        objData.ExecuteNonQuery(sqlStr)
    End Sub

    Public Sub addLocationInfoKap(ByVal infoID As String, ByVal kapNr As Integer, ByVal kapTittel As String, ByVal kapInfo As String, ByVal picURL As String)
        Dim sqlStr As String
        sqlStr = "INSERT INTO InfoKap (info_ID, kapNr, kapTittel, info, picURL) VALUES (" & infoID & ", " & kapNr & ", '" & kapTittel & "', '" & kapInfo & "', '" & picURL & "')"

        objData.ExecuteNonQuery(sqlStr)
    End Sub

    Public Sub addMap(ByVal mapURL As String, ByVal lokID As String)
        Dim sqlStr As String
        sqlStr = "INSERT INTO Kart (lok_ID, kartURL) VALUES (" & lokID & ", '" & mapURL & "')"

        objData.ExecuteNonQuery(sqlStr)
    End Sub

#End Region

#Region " ReInforceLocation " 'DEPRECATED
    Public Sub ReInforceLocation(ByVal fpID As String)
        Dim dtRow As DataRow
        Dim Sample_ID As Integer

        Dim SQLstr As String = "SELECT MAX(sample_ID) FROM AP WHERE posFP_ID = " & fpID

        Sample_ID = CInt(objData.ExecuteScalar(SQLstr))
        If Sample_ID.Equals(0) Then
            Sample_ID = 1
        Else
            Sample_ID += 1
        End If

        For Each dtRow In dtCurrentSignal.Rows
            'SQLstr = "INSERT INTO AP (MAC, Signal, Noise, SNR, Fingerprint_ID, Sample_ID) VALUES ('" _
            '& CStr(dtRow("MAC")) & "'," & CStr(dtRow("Signal")) & ", " & CStr(dtRow("Noise")) _
            '& ", " & CStr(dtRow("SNR")) & ", " & Fingerprint_ID & ", 1)"

            SQLstr = "INSERT INTO AP (MAC, signal, noise, posFP_ID, sample_ID) VALUES ('" & CStr(dtRow("MAC")) & "'," _
            & CStr(dtRow("Signal")) & ", " & CStr(dtRow("Noise")) & ", " & fpID & ", " & Sample_ID & ")"

            objData.ExecuteNonQuery(SQLstr)

            ' DEBUG
            'Console.WriteLine(SQLstr)
            'Console.WriteLine("Inserting...")
        Next
    End Sub
#End Region 'DEPRECATED

#Region " Remove "
    Public Sub Remove(ByRef ID As String)
        Dim SQLstr As String
        'Dim posFP_ID As String = getPosFP_ID(CStr(ID))
        SQLstr = "DELETE * FROM AP WHERE lok_ID = " & ID
        objData.ExecuteNonQuery(SQLstr)
        'Console.WriteLine(SQLstr)  ' DEBUG

        SQLstr = "DELETE * FROM PosFP WHERE lok_ID = " & ID
        objData.ExecuteNonQuery(SQLstr)
        'Console.WriteLine(SQLstr)  ' DEBUG
    End Sub

    Public Sub CurrentSignalClear()
        dtCurrentSignal.Rows.Clear()
    End Sub

#End Region

#Region " Get / Set "
    Public Function getCurrentSignal() As DataTable
        Return dtCurrentSignal
    End Function

    Public Function getSamplerFinished() As Boolean
        Return SamplerFinished
    End Function

    Public Function getLok(ByRef ID As String) As DataTable
        Dim SQLstr As String

        If ID.Equals("-1") Then
            SQLstr = "SELECT lokNavn, lok_ID FROM Lokasjon"
            'SQLstr = "SELECT L.lokNavn AS Navn, P.lok_ID AS lokID, FP.posFP_ID AS fpID FROM Lokasjon AS L, Posisjon AS P, PosFP AS FP WHERE L.lok_ID=P.lok_ID AND P.pos_ID=FP.pos_ID"
            'SQLstr = "SELECT FP.Name AS [Desc], R.Name AS [Room], F.Name AS [Floor], B.Name AS [Building], FP.Fingerprint_ID AS [ID], B.Name & ' ' & F.Name & ' ' & R.Name & ' ' & FP.Name AS [cName] FROM Fingerprint AS FP, Room AS R, Floor AS F, Building AS B WHERE FP.Room_ID=R.Room_ID And R.Floor_ID=F.Floor_ID And F.Building_ID=B.Building_ID"
            'Console.WriteLine("Loading..") ' DEBUG
        Else
            SQLstr = "SELECT lokNavn, lok_ID FROM Lokasjon WHERE lok_ID=" & ID
            'SQLstr = "SELECT L.lokNavn AS [Navn], FP.posFP_ID AS [ID] FROM Lokasjon AS L, PosFP AS FP WHERE FP.posFP_ID=" & ID & " L.lok_ID=Posisjon.lok_ID AND Posisjon.pos_ID=FP.pos_ID"
            'SQLstr = "SELECT FP.Name AS [Desc], R.Name AS [Room], F.Name AS [Floor], B.Name AS [Building], FP.Fingerprint_ID AS [ID], B.Name & ' ' & F.Name & ' ' & R.Name & ' ' & FP.Name AS [cName] FROM Fingerprint AS FP, Room AS R, Floor AS F, Building AS B WHERE FP.Fingerprint_ID=" & ID & " And FP.Room_ID=R.Room_ID And R.Floor_ID=F.Floor_ID And F.Building_ID=B.Building_ID"
        End If

        Return objData.ExecuteQuery(SQLstr)

    End Function

    Public Function getFPLok(ByRef FP_ID As String) As DataTable 'DEPRECATED
        Dim SQLstr As String
        SQLstr = "SELECT L.lokNavn, L.lok_ID FROM Lokasjon AS L, Posisjon AS P, PosFP AS FP WHERE L.lok_ID=P.lok_ID AND P.pos_ID=FP.pos_ID AND FP.posFP_ID=" & FP_ID
        Return objData.ExecuteQuery(SQLstr)
    End Function 'DEPRECATED

    Public Sub SetSamplerFinished()
        SamplerFinished = False
    End Sub

    Public Function getTopLevelNodes() As DataTable
        Dim SQLstr As String = "SELECT DISTINCT lokTypeNavn FROM LokType WHERE lokTypeNavn NOT IN (SELECT lokTypeNavnUnder FROM LokTypeRel)"
        Return objData.ExecuteQuery(SQLstr)
    End Function

    Public Function getChildNodes(ByVal lokType As String) As DataTable
        Dim SQLstr As String = "SELECT DISTINCT lokTypeNavnUnder FROM LokTypeRel WHERE lokTypeNavnOver = '" & lokType & "'"
        Return objData.ExecuteQuery(SQLstr)
    End Function

    Public Function getLokTypes() As DataTable 'Henter bare de som ligger under campus
        'Dim SQLstr As String = "SELECT lokTypeNavn FROM LokType Where NOT lokTypeNavn = 'Campus'"  'ACCESS
        Dim SQLstr As String = "SELECT lokTypeNavn FROM LokType Where lokTypeNavn <> 'Campus'"  'MySQL
        Dim dtLokTypes As DataTable
        dtLokTypes = objData.ExecuteQuery(SQLstr)
        Return dtLokTypes
    End Function

    Public Function getLokTypeFunk(ByVal lokType As String) As DataTable
        Dim SQLstr As String = "SELECT funkNavn FROM LokTypeFunk WHERE lokTypeNavn='" & lokType & "'"
        Dim dtLokTypeFunk As DataTable
        dtLokTypeFunk = objData.ExecuteQuery(SQLstr)
        Return dtLokTypeFunk
    End Function

    Public Function getFunk() As DataTable
        Dim SQLstr As String = "SELECT funkNavn FROM Funksjon"
        Dim dtFunk As DataTable
        dtFunk = objData.ExecuteQuery(SQLstr)
        Return dtFunk
    End Function

    Public Function getLokByType(ByVal lokType As String) As DataTable
        Dim SQLstr As String = "SELECT lokNavn, lok_ID FROM Lokasjon WHERE lokTypeNavn='" & lokType & "'"
        Dim dtLokByType As DataTable
        dtLokByType = objData.ExecuteQuery(SQLstr)
        Return dtLokByType
    End Function

    Public Function getSuperType(ByVal lokType As String) As String
        Dim SQLstr As String = "SELECT lokTypeNavnOver FROM LokTypeRel WHERE lokTypeNavnUnder='" & lokType & "'"
        Dim superType As String
        superType = objData.ExecuteScalar(SQLstr)
        Return superType
    End Function

    Public Function getChildType(ByVal lokType As String) As String
        Dim SQLstr As String = "SELECT lokTypeNavnUnder FROM LokTypeRel WHERE lokTypeNavnOver='" & lokType & "'"
        Dim childType As String
        childType = objData.ExecuteScalar(SQLstr)
        Return childType
    End Function

    Public Function getUsergroup(ByVal usertype As String) As DataTable
        Dim SQLstr As String = "SELECT brukergruppe_ID FROM Brukergruppe WHERE brukertype_ID='" & usertype & "'"
        Dim dtBrukergruppe As DataTable
        dtBrukergruppe = objData.ExecuteQuery(SQLstr)
        Return dtBrukergruppe
    End Function

    Public Function getUsertype() As DataTable
        Dim SQLstr As String = "SELECT brukertype_ID FROM Brukertype ORDER BY brukertype_ID ASC"
        Dim dtBrukertype As DataTable
        dtBrukertype = objData.ExecuteQuery(SQLstr)
        Return dtBrukertype
    End Function

    Public Function getInfotype() As DataTable
        Dim SQLstr As String = "SELECT infoArtNavn FROM InfoArt ORDER BY infoArtNavn ASC"
        Dim dtInfotype As DataTable
        dtInfotype = objData.ExecuteQuery(SQLstr)
        Return dtInfotype
    End Function

    Public Function getInfoKapNR(ByVal infoID As String) As String
        Dim SQLstr As String = "SELECT MAX(kapNr) FROM InfoKap WHERE info_ID =" & infoID
        Dim kapNr As String
        kapNr = objData.ExecuteScalar(SQLstr)
        Return kapNr
    End Function

    Public Function getInfoID() As String
        Dim SQLstr As String = "SELECT MAX(info_ID) FROM LokInfo"
        Dim infoID As String
        infoID = objData.ExecuteScalar(SQLstr)
        Return infoID
    End Function

    Public Function getFPcount(ByVal lokID As String) As String
        Dim SQLstr As String = "SELECT MAX(sample_ID) FROM AP WHERE lok_ID=" & lokID
        Dim FPcount As String
        FPcount = objData.ExecuteScalar(SQLstr)
        Return FPcount
    End Function

    Public Function getLokNavn(ByVal lokID As String) As String
        Dim SQLstr As String = "SELECT lokNavn FROM Lokasjon WHERE lok_ID=" & lokID
        Dim lokNavn As String
        lokNavn = objData.ExecuteScalar(SQLstr)
        Return lokNavn
    End Function

    Public Function getLokFunk(ByVal lokID As String) As String
        Dim SQLstr As String = "SELECT funkNavn FROM LokFunk WHERE lok_ID=" & lokID
        Dim lokFunk As String
        lokFunk = objData.ExecuteScalar(SQLstr)
        Return lokFunk
    End Function

    Public Function getLokType(ByVal lokID As String) As String
        Dim SQLstr As String = "SELECT lokTypeNavn FROM Lokasjon WHERE lok_ID=" & lokID
        Dim lokType As String
        lokType = objData.ExecuteScalar(SQLstr)
        Return lokType
    End Function

    Public Function getServiceURL(ByVal lokID As String) As String
        Dim SQLstr As String = "SELECT serviceURL FROM LokInfo WHERE lok_ID=" & lokID
        Dim serviceURL As String
        serviceURL = objData.ExecuteScalar(SQLstr)
        Return serviceURL
    End Function

    Public Function getLocImage(ByVal infoID As String, ByVal kapNr As String) As Image
        Dim objImage As Image
        Dim SQLstr As String = "SELECT picURL FROM infoKap WHERE info_ID=" & infoID & " AND kapNR=" & kapNr
        Dim picURL As String
        picURL = objData.ExecuteScalar(SQLstr)
        If picURL <> Nothing And picURL <> "0" Then
            objImage = New System.Drawing.Bitmap(New IO.MemoryStream(New System.Net.WebClient().DownloadData(picURL)))
            Return objImage
        Else
            Return Image.FromFile("gfx/no_image.gif")
        End If
    End Function

    Public Function getPosFP_ID(ByVal posID As Integer) As Integer 'DEPRECATED
        Dim posFP_ID As Integer
        Dim SQLstr As String

        If posID = -1 Then
            SQLstr = "SELECT COUNT(*) FROM PosFP"
            posFP_ID = CInt(objData.ExecuteScalar(SQLstr))
        Else
            SQLstr = "SELECT COUNT(*) FROM PosFP WHERE pos_ID=" & posID
            If CInt(objData.ExecuteScalar(SQLstr)) = 0 Then
                posFP_ID = 0
            Else
                SQLstr = "SELECT MAX(posFP_ID) FROM PosFP WHERE pos_ID=" & posID
                ' MAX instead of COUNT (deleted samples)
                posFP_ID = CInt(objData.ExecuteScalar(SQLstr))
            End If
        End If
        Return posFP_ID
    End Function 'DEPRECATED

    Public Function getPosID(ByVal lok_ID As Integer) As Integer 'DEPRECATED
        Dim pos_ID As Integer
        Dim SQLstr As String
        If lok_ID = -1 Then
            SQLstr = "SELECT MAX(pos_ID) FROM Posisjon"
            pos_ID = CInt(objData.ExecuteScalar(SQLstr))
        Else
            SQLstr = "SELECT pos_ID FROM Posisjon WHERE posType='FP' AND lok_ID=" & lok_ID
            pos_ID = CInt(objData.ExecuteScalar(SQLstr))

        End If
        Return pos_ID
    End Function 'DEPRECATED

    Public Function getLokID() As Integer
        Dim lok_ID As Integer
        Dim SQLstr As String = "SELECT COUNT(*) FROM Lokasjon"

        If CInt(objData.ExecuteScalar(SQLstr)) = 0 Then
            lok_ID = 0
        Else
            SQLstr = "SELECT MAX(lok_ID) FROM Lokasjon"
            ' MAX instead of COUNT (deleted samples)
            lok_ID = CInt(objData.ExecuteScalar(SQLstr))
        End If
        Return lok_ID
    End Function

    Public Function chkFP(ByVal lokID As Integer) As Boolean
        Dim exist As Integer
        Dim SQLstr As String = "SELECT COUNT(*) FROM PosFP WHERE lok_ID=" & lokID
        exist = CInt(objData.ExecuteScalar(SQLstr))

        If exist = 0 Then
            Return False
        Else
            Return True
        End If
    End Function

#End Region

#Region " Inference "
    Public Function Inference() As Integer(,)
        Dim i, j, NrOfSamples, Counter, NrOfApCurrent, NrOfApStored, APsFound, SampleScore As Integer
        Dim AP As DataTable
        'Dim dtRow As DataRow

        Dim SQLstr As String = "SELECT COUNT(lok_ID) FROM PosFP"
        ' MAX instead of COUNT (deleted samples)

        'Counter = CInt(objData.ExecuteScalar(SQLstr))
        'Dim Locs(Counter) As Integer

        SQLstr = "SELECT * FROM PosFP"
        Dim dtFP As DataTable = (objData.ExecuteQuery(SQLstr))
        Counter = dtFP.Rows.Count()
        Dim Score(Counter, 1) As Integer

        i = 1
        For Each dtRowFP As DataRow In dtFP.Rows
            Score(i, 0) = 999999
            Score(i, 1) = CInt(dtRowFP.Item("lok_ID"))

            NrOfSamples = CInt(objData.ExecuteScalar("SELECT MAX(sample_ID) FROM AP WHERE lok_ID = " & Score(i, 1)))

            For j = 1 To NrOfSamples
                For Each dtRowCS As DataRow In dtCurrentSignal.Rows
                    SQLstr = "SELECT * FROM AP WHERE lok_ID = " & Score(i, 1) & " AND MAC ='" & CStr(dtRowCS("MAC")) & "' AND sample_ID = " & j
                    AP = objData.ExecuteQuery(SQLstr)
                    If AP.Rows.Count = 0 Then
                        ' AP not found, worse score? Yes
                        SampleScore += 10
                    Else
                        SampleScore += Math.Abs(CInt(dtRowCS("Signal")) - CInt(AP.Rows(0)("Signal")))
                        APsFound += 1
                    End If
                Next ' For each dtRow in dtCurrentSignal

                ' NEW CODE
                SQLstr = "SELECT COUNT(*) FROM AP WHERE lok_ID = " & Score(i, 1) & " AND sample_ID = " & j

                NrOfApStored = CInt(objData.ExecuteScalar(SQLstr))

                If NrOfApStored > APsFound Then
                    SampleScore += (NrOfApStored - APsFound) * 5
                    ' If one or more AP's are missing in the sample +5 for each AP missing
                    If APsFound = 0 Then
                        SampleScore = 9999 ' If no AP match, this is NOT the location, BAD score
                    End If
                End If
                ' END NEW CODE

                If Score(i, 0) > SampleScore Then
                    Score(i, 0) = SampleScore
                End If
                SampleScore = 0 ' DJISES!! X 2 !! Reset scores
                APsFound = 0
            Next ' For sample 1 to NrOfSamples
            i += 1
        Next ' For 1 to last Fingerprint

        Return Score
    End Function
#End Region

#Region " Garbage "
    Public Sub insert()
        Dim SQLstr As String

        Dim i As Integer
        For i = 340 To 370
            SQLstr = "INSERT INTO Room (Name, Floor_ID) VALUES ('" & i & "', '" & 11 & "')"
            objData.ExecuteNonQuery(SQLstr)
        Next
    End Sub

    Public Sub insertCULBIS()
        Dim SQLstr(80) As String

        Dim i As Integer = 0

        SQLstr(i) = "INSERT INTO LokType VALUES ('Campus')" : i += 1
        SQLstr(i) = "INSERT INTO LokType VALUES ('Bygning')" : i += 1
        SQLstr(i) = "INSERT INTO LokType VALUES ('Etasje')" : i += 1
        SQLstr(i) = "INSERT INTO LokType VALUES ('Rom')" : i += 1
        'SQLstr(i) = "INSERT INTO LokType VALUES ('Fasilitet')" : i += 1
        SQLstr(i) = "INSERT INTO LokType VALUES ('Uteareal')" : i += 1
        SQLstr(i) = "INSERT INTO LokType VALUES ('Plass')" : i += 1
        SQLstr(i) = "INSERT INTO LokType VALUES ('Vei')" : i += 1

        'Delte
        SQLstr(i) = "INSERT INTO Funksjon VALUES ('Toalett')" : i += 1
        SQLstr(i) = "INSERT INTO Funksjon VALUES ('Kiosk')" : i += 1
        SQLstr(i) = "INSERT INTO Funksjon VALUES ('Kantine')" : i += 1
        SQLstr(i) = "INSERT INTO Funksjon VALUES ('Bibliotek')" : i += 1
        SQLstr(i) = "INSERT INTO Funksjon VALUES ('Uspes.')" : i += 1
        'Rom
        SQLstr(i) = "INSERT INTO Funksjon VALUES ('Datasal')" : i += 1
        SQLstr(i) = "INSERT INTO Funksjon VALUES ('Grupperom')" : i += 1
        SQLstr(i) = "INSERT INTO Funksjon VALUES ('Auditorium')" : i += 1
        SQLstr(i) = "INSERT INTO Funksjon VALUES ('Lesesal')" : i += 1
        SQLstr(i) = "INSERT INTO Funksjon VALUES ('Printerrom')" : i += 1
        SQLstr(i) = "INSERT INTO Funksjon VALUES ('Kopirom')" : i += 1
        SQLstr(i) = "INSERT INTO Funksjon VALUES ('Instituttkontor')" : i += 1
        SQLstr(i) = "INSERT INTO Funksjon VALUES ('Arbeidskontor')" : i += 1
        SQLstr(i) = "INSERT INTO Funksjon VALUES ('Arbeidsrom')" : i += 1
        'Fasilitet 
        'SQLstr(i) = "INSERT INTO Funksjon VALUES ('Brusautomat')" : i += 1
        'SQLstr(i) = "INSERT INTO Funksjon VALUES ('Kaffeautomat')" : i += 1
        'SQLstr(i) = "INSERT INTO Funksjon VALUES ('Printer')" : i += 1
        'SQLstr(i) = "INSERT INTO Funksjon VALUES ('Kopimaskin')" : i += 1

        'Plass
        SQLstr(i) = "INSERT INTO Funksjon VALUES ('Rasteplass')" : i += 1
        SQLstr(i) = "INSERT INTO Funksjon VALUES ('Parkeringsplass')" : i += 1

        SQLstr(i) = "INSERT INTO LokTypeRel (lokTypeNavnOver, lokTypeNavnUnder) VALUES ('Campus', 'Bygning')" : i += 1
        SQLstr(i) = "INSERT INTO LokTypeRel (lokTypeNavnOver, lokTypeNavnUnder) VALUES ('Bygning', 'Etasje')" : i += 1
        SQLstr(i) = "INSERT INTO LokTypeRel (lokTypeNavnOver, lokTypeNavnUnder) VALUES ('Etasje', 'Rom')" : i += 1
        'SQLstr(i) = "INSERT INTO LokTypeRel (lokTypeNavnOver, lokTypeNavnUnder) VALUES ('Rom', 'Fasilitet')" : i += 1
        SQLstr(i) = "INSERT INTO LokTypeRel (lokTypeNavnOver, lokTypeNavnUnder) VALUES ('Campus', 'Uteareal')" : i += 1
        SQLstr(i) = "INSERT INTO LokTypeRel (lokTypeNavnOver, lokTypeNavnUnder) VALUES ('Uteareal', 'Plass')" : i += 1
        SQLstr(i) = "INSERT INTO LokTypeRel (lokTypeNavnOver, lokTypeNavnUnder) VALUES ('Uteareal', 'Vei')" : i += 1

        SQLstr(i) = "INSERT INTO LokTypeFunk (lokTypeNavn, funkNavn) VALUES ('Bygning', 'Kiosk')" : i += 1
        SQLstr(i) = "INSERT INTO LokTypeFunk (lokTypeNavn, funkNavn) VALUES ('Bygning', 'Kantine')" : i += 1
        SQLstr(i) = "INSERT INTO LokTypeFunk (lokTypeNavn, funkNavn) VALUES ('Bygning', 'Bibliotek')" : i += 1
        SQLstr(i) = "INSERT INTO LokTypeFunk (lokTypeNavn, funkNavn) VALUES ('Bygning', 'Uspes.')" : i += 1
        SQLstr(i) = "INSERT INTO LokTypeFunk (lokTypeNavn, funkNavn) VALUES ('Etasje', 'Uspes.')" : i += 1
        SQLstr(i) = "INSERT INTO LokTypeFunk (lokTypeNavn, funkNavn) VALUES ('Rom', 'Datasal')" : i += 1
        SQLstr(i) = "INSERT INTO LokTypeFunk (lokTypeNavn, funkNavn) VALUES ('Rom', 'Grupperom')" : i += 1
        SQLstr(i) = "INSERT INTO LokTypeFunk (lokTypeNavn, funkNavn) VALUES ('Rom', 'Auditorium')" : i += 1
        SQLstr(i) = "INSERT INTO LokTypeFunk (lokTypeNavn, funkNavn) VALUES ('Rom', 'Lesesal')" : i += 1
        SQLstr(i) = "INSERT INTO LokTypeFunk (lokTypeNavn, funkNavn) VALUES ('Rom', 'Printerrom')" : i += 1
        SQLstr(i) = "INSERT INTO LokTypeFunk (lokTypeNavn, funkNavn) VALUES ('Rom', 'Kopirom')" : i += 1
        SQLstr(i) = "INSERT INTO LokTypeFunk (lokTypeNavn, funkNavn) VALUES ('Rom', 'Instituttkontor')" : i += 1
        SQLstr(i) = "INSERT INTO LokTypeFunk (lokTypeNavn, funkNavn) VALUES ('Rom', 'Arbeidskontor')" : i += 1
        SQLstr(i) = "INSERT INTO LokTypeFunk (lokTypeNavn, funkNavn) VALUES ('Rom', 'Arbeidsrom')" : i += 1
        SQLstr(i) = "INSERT INTO LokTypeFunk (lokTypeNavn, funkNavn) VALUES ('Rom', 'Toalett')" : i += 1
        SQLstr(i) = "INSERT INTO LokTypeFunk (lokTypeNavn, funkNavn) VALUES ('Rom', 'Kiosk')" : i += 1
        SQLstr(i) = "INSERT INTO LokTypeFunk (lokTypeNavn, funkNavn) VALUES ('Rom', 'Kantine')" : i += 1
        SQLstr(i) = "INSERT INTO LokTypeFunk (lokTypeNavn, funkNavn) VALUES ('Rom', 'Bibliotek')" : i += 1
        SQLstr(i) = "INSERT INTO LokTypeFunk (lokTypeNavn, funkNavn) VALUES ('Rom', 'Uspes.')" : i += 1
        'SQLstr(i) = "INSERT INTO LokTypeFunk (lokTypeNavn, funkNavn) VALUES ('Rom', '')" : i += 1
        SQLstr(i) = "INSERT INTO LokTypeFunk (lokTypeNavn, funkNavn) VALUES ('Uteareal', 'Uspes.')" : i += 1
        SQLstr(i) = "INSERT INTO LokTypeFunk (lokTypeNavn, funkNavn) VALUES ('Plass', 'Parkeringsplass')" : i += 1
        SQLstr(i) = "INSERT INTO LokTypeFunk (lokTypeNavn, funkNavn) VALUES ('Plass', 'Uspes.')" : i += 1
        SQLstr(i) = "INSERT INTO LokTypeFunk (lokTypeNavn, funkNavn) VALUES ('Vei', 'Uspes.')" : i += 1

        SQLstr(i) = "INSERT INTO Brukertype VALUES ('Ansatt')" : i += 1
        SQLstr(i) = "INSERT INTO Brukertype VALUES ('Student')" : i += 1
        SQLstr(i) = "INSERT INTO Brukertype VALUES ('Beskende')" : i += 1
        SQLstr(i) = "INSERT INTO Brukertype VALUES ('Arbeider')" : i += 1
        SQLstr(i) = "INSERT INTO Brukertype VALUES ('Alle')" : i += 1

        SQLstr(i) = "INSERT INTO Brukergruppe VALUES ('Informatikk', 'Student')" : i += 1
        SQLstr(i) = "INSERT INTO Brukergruppe VALUES ('Elektro', 'Student')" : i += 1
        SQLstr(i) = "INSERT INTO Brukergruppe VALUES ('Kybernetikk', 'Student')" : i += 1
        SQLstr(i) = "INSERT INTO Brukergruppe VALUES ('IDI', 'Ansatt')" : i += 1
        SQLstr(i) = "INSERT INTO Brukergruppe VALUES ('Vaktmester', 'Ansatt')" : i += 1
        SQLstr(i) = "INSERT INTO Brukergruppe VALUES ('Alle', 'Alle')" : i += 1

        SQLstr(i) = "INSERT INTO Kontekst VALUES ('Skole')" : i += 1
        SQLstr(i) = "INSERT INTO Kontekst VALUES ('Fritid')" : i += 1
        SQLstr(i) = "INSERT INTO Kontekst VALUES ('Arbeid')" : i += 1

        SQLstr(i) = "INSERT INTO Brukerprofil (brukernavn, passord, fnavn, enavn, kontekst_ID, brukergruppe_ID) VALUES ('bjorneha', 'passord', 'Bjrn Egil', 'Haugvik', 'Skole', 'Informatikk')" : i += 1
        SQLstr(i) = "INSERT INTO Brukerprofil (brukernavn, passord, fnavn, enavn, kontekst_ID, brukergruppe_ID) VALUES ('geir', 'pwd', 'Geir Marius', 'Gjul', 'Fritid', 'Informatikk')" : i += 1

        SQLstr(i) = "INSERT INTO Tidsform VALUES ('pningstid')" : i += 1
        SQLstr(i) = "INSERT INTO Tidsform VALUES ('Gyldighetstid')" : i += 1
        SQLstr(i) = "INSERT INTO Tidsform VALUES ('Varighet')" : i += 1
        SQLstr(i) = "INSERT INTO Tidsform VALUES ('Leveringstid')" : i += 1

        SQLstr(i) = "INSERT INTO InfoArt VALUES ('Reklame')" : i += 1
        SQLstr(i) = "INSERT INTO InfoArt VALUES ('Info')" : i += 1
        SQLstr(i) = "INSERT INTO InfoArt VALUES ('Meny')" : i += 1
        SQLstr(i) = "INSERT INTO InfoArt VALUES ('Kunngjring')" : i += 1

        'SQLstr(i) = "INSERT INTO PosType VALUES ('GPS')" : i += 1
        'SQLstr(i) = "INSERT INTO PosType VALUES ('FP')" : i += 1

        SQLstr(i) = "INSERT INTO RelType (relType_ID) VALUES ('del_av')" : i += 1
        SQLstr(i) = "INSERT INTO RelType (relType_ID) VALUES ('sees_fra')" : i += 1

        SQLstr(i) = "INSERT INTO CurrentSignal VALUES ('00146A171160', -35, -96)" : i += 1
        SQLstr(i) = "INSERT INTO CurrentSignal VALUES ('00146A171180', -62, -97)" : i += 1

        SQLstr(i) = "INSERT INTO Lokasjon (lok_ID, lokNavn, lokTypeNavn) VALUES (1, 'Glshaugen', 'Campus')" : i += 1

        For i = 0 To (SQLstr.Length - 1)
            objData.ExecuteNonQuery(SQLstr(i))
        Next
        Console.WriteLine("Testdata insertion completed!")
    End Sub
#End Region

#Region " Testing"
    Public Function testDB(ByVal sql As String) As String
        Dim SQLstr As String = sql
        Dim result As String
        result = objData.ExecuteScalar(SQLstr)
        Return result
    End Function
#End Region

#Region " Info PUSH "
    Public Function chkLokInfo(ByVal lokID As String) As Boolean
        Dim SQLstr As String = "SELECT info_ID FROM LokInfo WHERE lok_ID =" & lokID
        Dim chkInfo As String
        chkInfo = objData.ExecuteScalar(SQLstr)
        If chkInfo = "0" Then
            Return False
        Else
            Return True
        End If
    End Function

    Public Function getInfoID(ByVal lokID As String) As String
        Dim SQLstr As String = "SELECT info_ID FROM LokInfo WHERE lok_ID=" & lokID
        Dim infoID As String
        infoID = objData.ExecuteScalar(SQLstr)
        Return infoID
    End Function

    Public Function getKapInfo(ByVal infoID As String, ByVal kapNr As String) As String
        Dim SQLstr As String = "SELECT info FROM InfoKap WHERE info_ID=" & infoID & " AND kapNr=" & kapNr
        Dim kapInfo As String
        kapInfo = objData.ExecuteScalar(SQLstr)
        Return kapInfo
    End Function

    Public Function getKapTittel(ByVal infoID As String, ByVal kapNr As Integer) As String
        Dim SQLstr As String = "SELECT kapTittel FROM InfoKap WHERE info_ID=" & infoID & " AND kapNr=" & kapNr
        Dim kapTittel As String
        kapTittel = objData.ExecuteScalar(SQLstr)
        Return kapTittel
    End Function

    Public Function getMapURL(ByVal lokID As String) As String
        Dim SQLstr As String = "SELECT kartURL FROM Kart WHERE lok_ID =" & lokID
        Dim kartURL As String
        kartURL = objData.ExecuteScalar(SQLstr)
        Return kartURL
    End Function

    Public Function traceLoc(ByVal lokID As String) As String
        Dim traceArr() As String
        Dim trace As String
        Dim i As Integer = 0
        Dim ID As String = lokID
        While getSuperLoc(ID) <> Nothing
            ReDim Preserve traceArr(i)
            traceArr(i) = getLokNavn(ID)
            ID = getSuperLoc(ID)
            i += 1
        End While

        Dim j As Integer = traceArr.Length - 1
        While j >= 0
            If j = 0 Then
                trace += traceArr(j)
            Else
                trace += traceArr(j) & " > "
            End If
            j -= 1
        End While

        Return trace
    End Function

    Public Function getSuperLoc(ByVal lokID As String) As String
        Dim SQLstr As String = "SELECT lok_ID_til FROM Relasjon WHERE lok_ID_fra =" & lokID
        Dim superLoc As String
        superLoc = objData.ExecuteScalar(SQLstr)
        Return superLoc
    End Function

    Public Function getChildLocs(ByVal lokID As String) As DataTable
        Dim SQLstr As String = "SELECT lok_ID_fra AS lok_ID FROM Relasjon WHERE lok_ID_til =" & lokID
        Dim childLocs As DataTable
        childLocs = objData.ExecuteQuery(SQLstr)
        Return childLocs
    End Function

    Public Function findLokByFunk(ByVal currentLok As Integer, ByVal funk As String) As DataTable
        Dim SQLstr As String = "SELECT L.lokNavn AS lokNavn, L.lok_ID AS lok_ID FROM Lokasjon AS L, LokFunk AS F, Relasjon AS R WHERE R.lok_ID_til=" & currentLok & " And R.lok_ID_fra=L.lok_ID And L.lok_ID=F.lok_ID And F.funkNavn='" & funk & "'"
        Dim dtLocs As DataTable
        dtLocs = objData.ExecuteQuery(SQLstr)
        Return dtLocs
    End Function

    Public Function findLokByFunk(ByVal funk As String) As DataTable
        Dim SQLstr As String = "SELECT L.lokNavn AS lokNavn, L.lok_ID AS lok_ID FROM Lokasjon AS L, LokFunk AS F WHERE L.lok_ID=F.lok_ID And F.funkNavn='" & funk & "'"
        Dim dtLocs As DataTable
        dtLocs = objData.ExecuteQuery(SQLstr)
        Return dtLocs
    End Function

    Public Function getEvent(ByVal lokID As String) As DataTable
        Dim SQLstr As String = "SELECT E.eventTitle AS Title, E.eventDescr AS Descr FROM Event E, Lokasjon L WHERE E.lok_ID=L.lok_ID AND L.lok_ID=" & lokID
        Dim dtEvent As DataTable
        dtEvent = objData.ExecuteQuery(SQLstr)
        Return dtEvent
    End Function

#End Region

#Region " Info INTERACT "
    Public Sub annotate(ByVal username As String, ByVal lokID As Integer, ByVal title As String, ByVal annotation As String)
        Dim SQLstr As String = "INSERT INTO Annotasjon (lok_ID, brukernavn, annoNavn, annotasjon) VALUES (" _
        & lokID & ", '" & username & "', '" & title & "', '" & annotation & "')"
        objData.ExecuteNonQuery(SQLstr)
    End Sub

    Public Function getAnnotations(ByVal username As String, ByVal lokID As Integer) As String
        Dim annotation As String
        Dim SQLstr As String = "SELECT annoNavn, annotasjon FROM Annotasjon WHERE brukernavn='" & username & "' AND lok_ID=" & lokID
        Dim dtAnno As DataTable = objData.ExecuteQuery(SQLstr)
        For Each dtRow As DataRow In dtAnno.Rows
            annotation += CStr(dtRow.Item("annoNavn")) + vbNewLine + CStr(dtRow.Item("annotasjon")) + vbNewLine + vbNewLine
        Next
        Return annotation
    End Function

    Public Function get2doList(ByVal username As String, ByVal lokID As Integer) As DataTable
        Dim todoList As String

        Dim SQLstr As String = "SELECT T.todo_ID AS todoID, E.entry_ID AS entryID, E.entryInfo as entryInfo FROM 2do T, 2doEntry E WHERE T.brukernavn='" & username & "' AND T.lok_ID=" & lokID & " AND T.todo_ID=E.todo_ID"
        Dim dt2do As DataTable = objData.ExecuteQuery(SQLstr)

        'For Each dtRow As DataRow In dtAnno.Rows
        'annotation += CStr(dtRow.Item("annoNavn")) + vbNewLine + CStr(dtRow.Item("annotasjon")) + vbNewLine + vbNewLine
        'Next
        If dt2do.Rows.Count = 0 Then
            Return Nothing
        Else
            Return dt2do
        End If
    End Function

    Public Sub add2do(ByVal descr As String, ByVal username As String, ByVal lokID As String)
        Dim SQLstr As String = "SELECT todo_ID FROM 2do WHERE brukernavn='" & username & "' AND lok_ID=" & lokID
        Dim todoID As String = objData.ExecuteScalar(SQLstr)

        If todoID <> Nothing Then
            SQLstr = "INSERT INTO 2doEntry (entryInfo, todo_ID) VALUES ('" & descr & "', " & todoID & ")"
            objData.ExecuteNonQuery(SQLstr)
        Else
            SQLstr = "INSERT INTO 2do (brukernavn, lok_ID) VALUES ('" & username & "', " & lokID & ")"
            objData.ExecuteNonQuery(SQLstr)
            SQLstr = "SELECT todo_ID FROM 2do WHERE brukernavn='" & username & "' AND lok_ID=" & lokID
            todoID = objData.ExecuteScalar(SQLstr)
            SQLstr = "INSERT INTO 2doEntry (entryInfo, todo_ID) VALUES ('" & descr & "', " & todoID & ")"
            objData.ExecuteNonQuery(SQLstr)
        End If
    End Sub

    Public Sub remove2do(ByVal entryID As String, ByVal username As String, ByVal lokID As String)
        Dim SQLstr As String = "DELETE * FROM 2doEntry WHERE entry_ID=" & entryID
        objData.ExecuteNonQuery(SQLstr) 'Sletter post

        SQLstr = "SELECT todo_ID FROM 2do WHERE brukernavn='" & username & "' AND lok_ID=" & lokID
        Dim todoID As String = objData.ExecuteScalar(SQLstr)

        SQLstr = "SELECT entry_ID FROM 2doEntry WHERE todo_ID=" & todoID
        Dim chkEntries As String = objData.ExecuteScalar(SQLstr)

        If chkEntries = Nothing Then 'Sletter listen hvis det ikke finnes noen entries
            SQLstr = "DELETE * FROM 2do WHERE todo_ID=" & todoID
            objData.ExecuteNonQuery(SQLstr)
        End If
    End Sub
#End Region

#Region " Info PULL "
    Public Function search(ByVal searchstring As String, ByVal searchType As String) As DataTable ' IN PROGRESS
        Dim result As DataTable
        Dim SQLstr As String
        'Hardcoding for  unng at lok_ID 1 blir funnet da det gir krasj i traceLoc funksjonen
        If searchType = "Beskrivelse" Then
            SQLstr = "SELECT DISTINCT L.lok_ID AS lok_ID, L.lokNavn AS lokNavn FROM Lokasjon L, LokInfo LI, InfoKap K WHERE LI.info_ID=K.info_ID AND L.lok_ID=LI.lok_ID AND L.lok_ID <> 1 AND K.info LIKE '%" & searchstring & "%'"
            result = objData.ExecuteQuery(SQLstr)
            For Each dtRow As DataRow In result.Rows
                dtRow.Item("lokNavn") = traceLoc(CStr(dtRow.Item("lok_ID")))
            Next
        End If

        If searchType = "Navn" Then
            SQLstr = "SELECT L.lok_ID AS lok_ID, L.lokNavn AS lokNavn FROM Lokasjon L WHERE L.lok_ID <> 1 AND L.lokNavn LIKE '%" & searchstring & "%'"
            result = objData.ExecuteQuery(SQLstr)
            For Each dtRow As DataRow In result.Rows
                dtRow.Item("lokNavn") = traceLoc(CStr(dtRow.Item("lok_ID")))
            Next
        End If

        'For Each dtRow As DataRow In result2.Rows
        'result.Rows.Add(dtRow) 'NEED TO SOMEHOW MERGE THE RESULTS!!!!!!!!!!!!!!
        'Next

        Return result
    End Function
#End Region

#Region " LoggInn "
    Public Function loggInn(ByVal username As String, ByVal pass As String) As User
        Dim brukeren As User
        Dim brukertabell As DataTable = objData.ExecuteQuery("SELECT * FROM brukerprofil WHERE brukernavn = '" & username & "' AND passord = '" & pass & "'")
        Dim dtRow As DataRow
        Dim fnavn, enavn, brukertype, profiltype As String
        Dim found As Boolean
        found = False

        For Each dtRow In brukertabell.Rows
            found = True
            fnavn = CStr(dtRow.Item("fnavn"))
            enavn = CStr(dtRow.Item("enavn"))
            profiltype = CStr(dtRow.Item("kontekst_ID"))
            brukertype = CStr(dtRow.Item("brukergruppe_ID"))
            'prioritet = CStr(dtRow.Item("prioritet_ID"))
        Next

        If found Then
            brukeren = New User(username, fnavn, enavn, brukertype, profiltype)
        End If
        Return brukeren
    End Function

#End Region
End Class
